@@ -1,5 +1,5 @@
 /* tpb - program to use the IBM ThinkPad(tm) special keys
- * Copyright (C) 2002-2004 Markus Braun <markus.braun@krawel.de>
+ * Copyright (C) 2002-2005 Markus Braun <markus.braun@krawel.de>
  *
  * This file is part of tpb.
  *
@@ -42,7 +42,7 @@
 
 /* RCS version string for later identification using rcs ident {{{*/
 #ifndef lint
-static volatile const char *RCSid = "@(#) $Id: cfg.c,v 1.14 2004/08/18 13:57:32 mbr Exp $";
+static volatile const char *RCSid = "@(#) $Id: cfg.c,v 1.15 2005/07/18 14:15:59 mbr Exp $";
 #endif /* lint */
 /* }}} */
 
@@ -416,6 +416,27 @@ void init_cfg(config *cfg) { /* {{{ */
   /* cfg->osd */
   cfg->osd = DEFAULT_OSD;
 
+  /* cfg->osdzoom */
+  cfg->osdzoom = DEFAULT_OSDZOOM;
+
+  /* cfg->osdthinklight */
+  cfg->osdthinklight = DEFAULT_OSDTHINKLIGHT;
+
+  /* cfg->osddisplay */
+  cfg->osddisplay = DEFAULT_OSDDISPLAY;
+
+  /* cfg->osdhvexpansion */
+  cfg->osdhvexpansion = DEFAULT_OSDHVEXPANSION;
+
+  /* cfg->osdbrightness */
+  cfg->osdbrightness = DEFAULT_OSDBRIGHTNESS;
+
+  /* cfg->osdvolume */
+  cfg->osdvolume = DEFAULT_OSDVOLUME;
+
+  /* cfg->osdpowermgt */
+  cfg->osdpowermgt = DEFAULT_OSDPOWERMGT;
+
   /* cfg->osdfont */
   if(cfg->osdfont != NULL) {
     free(cfg->osdfont);
@@ -541,6 +562,27 @@ void clear_cfg(config *cfg) { /* {{{ */
   /* cfg->osd */
   cfg->osd = UNDEFINED;
 
+  /* cfg->osdzoom */
+  cfg->osdzoom = UNDEFINED;
+
+  /* cfg->osdthinklight */
+  cfg->osdthinklight = UNDEFINED;
+
+  /* cfg->osddisplay */
+  cfg->osddisplay = UNDEFINED;
+
+  /* cfg->osdhvexpansion */
+  cfg->osdhvexpansion = UNDEFINED;
+
+  /* cfg->osdbrightness */
+  cfg->osdbrightness = UNDEFINED;
+
+  /* cfg->osdvolume */
+  cfg->osdvolume = UNDEFINED;
+
+  /* cfg->osdpowermgt */
+  cfg->osdpowermgt = UNDEFINED;
+
   /* cfg->osdfont */
   cfg->osdfont = NULL;
 
@@ -831,6 +873,41 @@ void override_cfg(config *master, config *slave) { /* {{{ */
     slave->osd = master->osd;
   }
 
+  /* cfg->osdzoom */
+  if(master->osdzoom != UNDEFINED) {
+    slave->osdzoom = master->osdzoom;
+  }
+
+  /* cfg->osdthinklight */
+  if(master->osdthinklight != UNDEFINED) {
+    slave->osdthinklight = master->osdthinklight;
+  }
+
+  /* cfg->osddisplay */
+  if(master->osddisplay != UNDEFINED) {
+    slave->osddisplay = master->osddisplay;
+  }
+
+  /* cfg->osdhvexpansion */
+  if(master->osdhvexpansion != UNDEFINED) {
+    slave->osdhvexpansion = master->osdhvexpansion;
+  }
+
+  /* cfg->osdbrightness */
+  if(master->osdbrightness != UNDEFINED) {
+    slave->osdbrightness = master->osdbrightness;
+  }
+
+  /* cfg->osdvolume */
+  if(master->osdvolume != UNDEFINED) {
+    slave->osdvolume = master->osdvolume;
+  }
+
+  /* cfg->osdpowermgt */
+  if(master->osdpowermgt != UNDEFINED) {
+    slave->osdpowermgt = master->osdpowermgt;
+  }
+
   /* cfg->osdfont */
   if(master->osdfont != NULL) {
     if(slave->osdfont != NULL) {
@@ -1315,6 +1392,146 @@ void set_value(char *key, char *arg, config *cfg) { /* {{{ */
       }
     }
   } /* }}} */
+
+  else if (strcmp(CFG_OSDZOOM, key) == 0) { /* {{{ */
+    int i = 0;
+    while(arg[i] != '\0') {
+      arg[i] = tolower(arg[i]);
+      i++;
+    }
+    if(strcmp(CFG_OSDZOOM_ON, arg) == 0) {
+      cfg->osdzoom = STATE_ON;
+    }
+    else {
+      if(strcmp(CFG_OSDZOOM_OFF, arg) == 0) {
+	cfg->osdzoom = STATE_OFF;
+      }
+      else {
+	fprintf(stderr, _("Illegal xosd zoom state: %s\n"), arg);
+	_exit(1);
+      }
+    }
+  } /* }}} */
+
+  else if (strcmp(CFG_OSDTHINKLIGHT, key) == 0) { /* {{{ */
+    int i = 0;
+    while(arg[i] != '\0') {
+      arg[i] = tolower(arg[i]);
+      i++;
+    }
+    if(strcmp(CFG_OSDTHINKLIGHT_ON, arg) == 0) {
+      cfg->osdthinklight = STATE_ON;
+    }
+    else {
+      if(strcmp(CFG_OSDTHINKLIGHT_OFF, arg) == 0) {
+	cfg->osdthinklight = STATE_OFF;
+      }
+      else {
+	fprintf(stderr, _("Illegal xosd thinklight state: %s\n"), arg);
+	_exit(1);
+      }
+    }
+  } /* }}} */
+
+  else if (strcmp(CFG_OSDDISPLAY, key) == 0) { /* {{{ */
+    int i = 0;
+    while(arg[i] != '\0') {
+      arg[i] = tolower(arg[i]);
+      i++;
+    }
+    if(strcmp(CFG_OSDDISPLAY_ON, arg) == 0) {
+      cfg->osddisplay = STATE_ON;
+    }
+    else {
+      if(strcmp(CFG_OSDDISPLAY_OFF, arg) == 0) {
+	cfg->osddisplay = STATE_OFF;
+      }
+      else {
+	fprintf(stderr, _("Illegal xosd display state: %s\n"), arg);
+	_exit(1);
+      }
+    }
+  } /* }}} */
+
+  else if (strcmp(CFG_OSDHVEXPANSION, key) == 0) { /* {{{ */
+    int i = 0;
+    while(arg[i] != '\0') {
+      arg[i] = tolower(arg[i]);
+      i++;
+    }
+    if(strcmp(CFG_OSDHVEXPANSION_ON, arg) == 0) {
+      cfg->osdhvexpansion = STATE_ON;
+    }
+    else {
+      if(strcmp(CFG_OSDHVEXPANSION_OFF, arg) == 0) {
+	cfg->osdhvexpansion = STATE_OFF;
+      }
+      else {
+	fprintf(stderr, _("Illegal xosd HV expansion state: %s\n"), arg);
+	_exit(1);
+      }
+    }
+  } /* }}} */
+
+  else if (strcmp(CFG_OSDBRIGHTNESS, key) == 0) { /* {{{ */
+    int i = 0;
+    while(arg[i] != '\0') {
+      arg[i] = tolower(arg[i]);
+      i++;
+    }
+    if(strcmp(CFG_OSDBRIGHTNESS_ON, arg) == 0) {
+      cfg->osdbrightness = STATE_ON;
+    }
+    else {
+      if(strcmp(CFG_OSDBRIGHTNESS_OFF, arg) == 0) {
+	cfg->osdbrightness = STATE_OFF;
+      }
+      else {
+	fprintf(stderr, _("Illegal xosd brightness state: %s\n"), arg);
+	_exit(1);
+      }
+    }
+  } /* }}} */
+
+  else if (strcmp(CFG_OSDVOLUME, key) == 0) { /* {{{ */
+    int i = 0;
+    while(arg[i] != '\0') {
+      arg[i] = tolower(arg[i]);
+      i++;
+    }
+    if(strcmp(CFG_OSDVOLUME_ON, arg) == 0) {
+      cfg->osdvolume = STATE_ON;
+    }
+    else {
+      if(strcmp(CFG_OSDVOLUME_OFF, arg) == 0) {
+	cfg->osdvolume = STATE_OFF;
+      }
+      else {
+	fprintf(stderr, _("Illegal xosd volume state: %s\n"), arg);
+	_exit(1);
+      }
+    }
+  } /* }}} */
+
+  else if (strcmp(CFG_OSDPOWERMGT, key) == 0) { /* {{{ */
+    int i = 0;
+    while(arg[i] != '\0') {
+      arg[i] = tolower(arg[i]);
+      i++;
+    }
+    if(strcmp(CFG_OSDPOWERMGT_ON, arg) == 0) {
+      cfg->osdpowermgt = STATE_ON;
+    }
+    else {
+      if(strcmp(CFG_OSDPOWERMGT_OFF, arg) == 0) {
+	cfg->osdpowermgt = STATE_OFF;
+      }
+      else {
+	fprintf(stderr, _("Illegal xosd powermanagement state: %s\n"), arg);
+	_exit(1);
+      }
+    }
+  } /* }}} */
 #endif /* HAVE_LIBXOSD */
 
   return;
@@ -1349,7 +1566,7 @@ void find_nvram(config *cfg) /* {{{ */
 /* print the usage to sdtout */
 void print_usage(const char *prog) /* {{{ */
 {
-  printf("%s " VERSION " (C) 2002-2004 Markus Braun \n\n", prog);
+  printf("%s " VERSION " (C) 2002-2005 Markus Braun \n\n", prog);
   printf(_("Usage: %s options\n\n"), prog);
   printf(_("options:\n"));
   printf(_("   -h, --help           display this help\n"));
