@@ -1,5 +1,5 @@
 /* tpb - program to use the IBM ThinkPad(tm) special keys
- * Copyright (C) 2002-2004 Markus Braun <markus.braun@krawel.de>
+ * Copyright (C) 2002-2005 Markus Braun <markus.braun@krawel.de>
  *
  * This file is part of tpb.
  *
@@ -52,7 +52,7 @@
 
 /* RCS version string for later identification using rcs ident {{{ */
 #ifndef lint
-static volatile const char *RCSid = "@(#) $Id: tpb.c,v 1.29 2004/08/22 13:45:11 mbr Exp $";
+static volatile const char *RCSid = "@(#) $Id: tpb.c,v 1.31 2005/07/18 14:15:59 mbr Exp $";
 #endif /* lint */
 /* }}} */
 
@@ -148,9 +148,7 @@ int main(int argc, char **argv) /* {{{ */
 
   /* initialize osd */
 #ifdef HAVE_LIBXOSD
-  if(cfg.osd == STATE_ON) {
-    osd_ptr = init_xosd();
-  }
+  osd_ptr = init_xosd();
 #endif /* HAVE_LIBXOSD */
 
 #ifdef HAVE_LIBX11
@@ -239,7 +237,8 @@ int main(int argc, char **argv) /* {{{ */
 	}
       }
 #ifdef HAVE_LIBXOSD
-      if(osd_ptr != NULL) {
+      if(osd_ptr != NULL &&
+	 ((cfg.osd == STATE_OFF && cfg.osdzoom == STATE_ON) || (cfg.osd == STATE_ON && cfg.osdzoom != STATE_OFF))) {
 	xosd_display(osd_ptr, 0, XOSD_string, thinkpad_state.zoom_toggle == 1 ? _("Zoom on") : _("Zoom off"));
 	xosd_display(osd_ptr, 1, XOSD_string, "");
       }
@@ -442,7 +441,8 @@ int main(int argc, char **argv) /* {{{ */
 	}
       }
 #ifdef HAVE_LIBXOSD
-      if(osd_ptr != NULL) {
+      if(osd_ptr != NULL &&
+	 ((cfg.osd == STATE_OFF && cfg.osdthinklight == STATE_ON) || (cfg.osd == STATE_ON && cfg.osdthinklight != STATE_OFF))) {
 	xosd_display(osd_ptr, 0, XOSD_string, thinkpad_state.thinklight_toggle == 1 ? _("ThinkLight on") : _("ThinkLight off"));
 	xosd_display(osd_ptr, 1, XOSD_string, "");
       }
@@ -492,7 +492,8 @@ int main(int argc, char **argv) /* {{{ */
 	}
       }
 #ifdef HAVE_LIBXOSD
-      if(osd_ptr != NULL) {
+      if(osd_ptr != NULL &&
+	 ((cfg.osd == STATE_OFF && cfg.osddisplay == STATE_ON) || (cfg.osd == STATE_ON && cfg.osddisplay != STATE_OFF))) {
 	xosd_display(osd_ptr, 0, XOSD_string, _("Display"));
 	xosd_display(osd_ptr, 1, XOSD_string, msg);
       }
@@ -511,7 +512,8 @@ int main(int argc, char **argv) /* {{{ */
 	}
       }
 #ifdef HAVE_LIBXOSD
-      if(osd_ptr != NULL) {
+      if(osd_ptr != NULL &&
+	 ((cfg.osd == STATE_OFF && cfg.osdhvexpansion == STATE_ON) || (cfg.osd == STATE_ON && cfg.osdhvexpansion != STATE_OFF))) {
 	xosd_display(osd_ptr, 0, XOSD_string, thinkpad_state.expand_toggle == 1 ? _("HV Expansion on") : _("HV Expansion off"));
 	xosd_display(osd_ptr, 1, XOSD_string, "");
       }
@@ -532,7 +534,8 @@ int main(int argc, char **argv) /* {{{ */
     }
 #ifdef HAVE_LIBXOSD
     if(thinkpad_state.brightness_toggle != last_thinkpad_state.brightness_toggle) {
-      if(osd_ptr != NULL) {
+      if(osd_ptr != NULL &&
+	 ((cfg.osd == STATE_OFF && cfg.osdbrightness == STATE_ON) || (cfg.osd == STATE_ON && cfg.osdbrightness != STATE_OFF))) {
 	xosd_display(osd_ptr, 0, XOSD_string, _("Brightness"));
 	xosd_display(osd_ptr, 1, XOSD_percentage, thinkpad_state.brightness_level * 100 / 7);
       }
@@ -569,7 +572,8 @@ int main(int argc, char **argv) /* {{{ */
     /* show volume every time a volume button is pressed and not mute */
     if(thinkpad_state.volume_toggle != last_thinkpad_state.volume_toggle &&
        thinkpad_state.mute_toggle == 0) {
-      if(osd_ptr != NULL) {
+      if(osd_ptr != NULL &&
+	 ((cfg.osd == STATE_OFF && cfg.osdvolume == STATE_ON) || (cfg.osd == STATE_ON && cfg.osdvolume != STATE_OFF))) {
 	xosd_display(osd_ptr, 0, XOSD_string, _("Volume"));
 	xosd_display(osd_ptr, 1, XOSD_percentage, vol);
       }
@@ -600,7 +604,8 @@ int main(int argc, char **argv) /* {{{ */
 	}
       }
 
-      if(osd_ptr != NULL) {
+      if(osd_ptr != NULL &&
+	 ((cfg.osd == STATE_OFF && cfg.osdvolume == STATE_ON) || (cfg.osd == STATE_ON && cfg.osdvolume != STATE_OFF))) {
 	if(thinkpad_state.mute_toggle == 1) {
 	  xosd_display(osd_ptr, 0, XOSD_string, _("Mute on"));
 	  xosd_display(osd_ptr, 1, XOSD_percentage, 0);
@@ -625,7 +630,8 @@ int main(int argc, char **argv) /* {{{ */
 	}
       }
 #ifdef HAVE_LIBXOSD
-      if(osd_ptr != NULL) {
+      if(osd_ptr != NULL &&
+	 ((cfg.osd == STATE_OFF && cfg.osdpowermgt == STATE_ON) || (cfg.osd == STATE_ON && cfg.osdpowermgt != STATE_OFF))) {
 	xosd_display(osd_ptr, 0, XOSD_string, thinkpad_state.ac_state == 1 ? _("AC connected") : _("AC disconnected"));
 	xosd_display(osd_ptr, 1, XOSD_string, "");
       }
@@ -663,7 +669,8 @@ int main(int argc, char **argv) /* {{{ */
 	}
       }
 #ifdef HAVE_LIBXOSD
-      if(osd_ptr != NULL) {
+      if(osd_ptr != NULL &&
+	 ((cfg.osd == STATE_OFF && cfg.osdpowermgt == STATE_ON) || (cfg.osd == STATE_ON && cfg.osdpowermgt != STATE_OFF))) {
 	if (cfg.powermgt == STATE_ON ||  (cfg.powermgt == STATE_AUTO && apmiser_running() == 0)) {
 	  xosd_display(osd_ptr, 0, XOSD_string, msg);
 	  xosd_display(osd_ptr, 1, XOSD_string, "");
@@ -703,7 +710,8 @@ int main(int argc, char **argv) /* {{{ */
 	}
       }
 #ifdef HAVE_LIBXOSD
-      if(osd_ptr != NULL) {
+      if(osd_ptr != NULL &&
+	 ((cfg.osd == STATE_OFF && cfg.osdpowermgt == STATE_ON) || (cfg.osd == STATE_ON && cfg.osdpowermgt != STATE_OFF))) {
 	if (cfg.powermgt == STATE_ON ||  (cfg.powermgt == STATE_AUTO && apmiser_running() == 0)) {
 	  xosd_display(osd_ptr, 0, XOSD_string, msg);
 	  xosd_display(osd_ptr, 1, XOSD_string, "");
@@ -1249,9 +1257,10 @@ int fork_app(char * cmd) /* {{{ */
       }
 
       setsid(); /* children should not be killed if tpb ends */
+      chdir(getenv("HOME")); /* try to change to users home directory */
       execv(args[0], args);
 
-      /* shold never be reached, execv only returns on error */
+      /* should never be reached, execv only returns on error */
       fprintf(stderr, _("Unable to fork application \"%s\".\n"), args[0]); 
       fprintf(stderr, _("HINT:\n")); 
       fprintf(stderr, _("With tpb version 0.6.2 or later you need to specify applications\n")); 
